@@ -1,9 +1,9 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2001, 2005, 2006 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2001, 2005, 2006, 2007 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
+   the Free Software Foundation; either version 3, or (at your option)
    any later version.
 
    GNU Mailutils is distributed in the hope that it will be useful,
@@ -280,7 +280,7 @@ fetch_fast (struct fetch_command *command, char **arg)
    Header: Date, Subject, From, Sender, Reply-To, To, Cc, Bcc, In-Reply-To,
    and Message-Id.  */
 static int
-fetch_envelope (struct fetch_command *command, char **arg ARG_UNUSED)
+fetch_envelope (struct fetch_command *command, char **arg MU_ARG_UNUSED)
 {
   int status;
   util_send ("%s (", command->name);
@@ -347,29 +347,26 @@ fetch_flags (struct fetch_command *command, char **arg)
    ;; from the given time will give the UT form.
    ;; The Universal Time zone is "+0000".  */
 static int
-fetch_internaldate (struct fetch_command *command, char **arg ARG_UNUSED)
+fetch_internaldate (struct fetch_command *command, char **arg MU_ARG_UNUSED)
 {
-  char date[128];
+  const char *date;
   mu_envelope_t env = NULL;
   struct tm tm, *tmp = NULL;
   mu_timezone tz;
+  char datebuf[sizeof ("13-Jul-2002 00:00:00")];
 
   mu_message_get_envelope (command->msg, &env);
-  date[0] = '\0';
-  if (mu_envelope_date (env, date, sizeof (date), NULL) == 0)
+  if (mu_envelope_sget_date (env, &date) == 0
+      && mu_parse_ctime_date_time (&date, &tm, &tz) == 0)
+    tmp = &tm;
+  else
     {
-      char *p = date;
-      if (mu_parse_ctime_date_time ((const char **) &p, &tm, &tz) == 0)
-	tmp = &tm;
+      time_t t = time (NULL);
+      tmp = localtime (&t);
     }
-  if (!tmp)
-    {
-      time_t t = time(NULL);
-      tmp = localtime(&t);
-    }
-  mu_strftime (date, sizeof (date), "%d-%b-%Y %H:%M:%S", tmp);
+  mu_strftime (datebuf, sizeof (datebuf), "%d-%b-%Y %H:%M:%S", tmp);
   util_send ("%s", command->name);
-  util_send (" \"%s +0000\"", date);
+  util_send (" \"%s +0000\"", datebuf);
   return RESP_OK;
 }
 
@@ -378,7 +375,7 @@ fetch_internaldate (struct fetch_command *command, char **arg ARG_UNUSED)
   Functionally equivalent to BODY.PEEK[HEADER], differing in the syntax of
   the resulting untagged FETCH data (RFC822.HEADER is returned). */
 static int
-fetch_rfc822_header (struct fetch_command *command, char **arg ARG_UNUSED)
+fetch_rfc822_header (struct fetch_command *command, char **arg MU_ARG_UNUSED)
 {
   char buffer[32];
   char *p = buffer;
@@ -391,7 +388,7 @@ fetch_rfc822_header (struct fetch_command *command, char **arg ARG_UNUSED)
    Functionally equivalent to BODY[TEXT], differing in the syntax of the
    resulting untagged FETCH data (RFC822.TEXT is returned). */
 static int
-fetch_rfc822_text (struct fetch_command *command, char **arg ARG_UNUSED)
+fetch_rfc822_text (struct fetch_command *command, char **arg MU_ARG_UNUSED)
 {
   char buffer[16];
   char *p = buffer;
@@ -402,7 +399,7 @@ fetch_rfc822_text (struct fetch_command *command, char **arg ARG_UNUSED)
 
 /* The [RFC-822] size of the message.  */
 static int
-fetch_rfc822_size (struct fetch_command *command, char **arg ARG_UNUSED)
+fetch_rfc822_size (struct fetch_command *command, char **arg MU_ARG_UNUSED)
 {
   size_t size = 0;
   size_t lines = 0;
@@ -457,7 +454,7 @@ fetch_rfc822 (struct fetch_command *command, char **arg)
 
 /* UID: The unique identifier for the message.  */
 static int
-fetch_uid (struct fetch_command *command, char **arg ARG_UNUSED)
+fetch_uid (struct fetch_command *command, char **arg MU_ARG_UNUSED)
 {
   size_t uid = 0;
 
@@ -471,7 +468,7 @@ fetch_uid (struct fetch_command *command, char **arg ARG_UNUSED)
    server by parsing the [MIME-IMB] header fields in the [RFC-822] header and
    [MIME-IMB] headers.  */
 static int
-fetch_bodystructure (struct fetch_command *command, char **arg ARG_UNUSED)
+fetch_bodystructure (struct fetch_command *command, char **arg MU_ARG_UNUSED)
 {
   util_send ("%s (", command->name);
   fetch_bodystructure0 (command->msg, 1); /* 1 means with extension data.  */
@@ -1034,7 +1031,7 @@ fetch_operation (mu_message_t msg, char **arg, int silent)
     {
       if (!silent)
 	{
-	  /* NOTE: We violate the RFC here: Header can not take a prefix for
+	  /* NOTE: We violate the RFC here: Header cannot take a prefix for
 	     section messages it only referes to the RFC822 header .. ok
 	     see it as an extension. But according to IMAP4 we should
 	     have send an empty string: util_send (" \"\"");
@@ -1160,7 +1157,7 @@ fetch_io (mu_stream_t stream, unsigned long start, unsigned long end,
       offset = 0;
       if (max)
 	{
-	  util_send (" {%u}\r\n", max);
+	  util_send (" {%lu}\r\n", max);
 	  while (mu_stream_read (rfc, buffer, sizeof (buffer) - 1, offset,
 			      &n) == 0 && n > 0)
 	    {
@@ -1433,8 +1430,6 @@ fetch_header_fields_not (mu_message_t msg, char **arg, unsigned long start,
   return RESP_OK;
 }
 
-/* FIXME: The address is limit by a buffer of 128, no good.  We should
-   allocate the buffer.  */
 static int
 fetch_send_address (const char *addr)
 {
@@ -1458,42 +1453,36 @@ fetch_send_address (const char *addr)
       return RESP_OK;
     }
 
-  util_send ("(", count);
+  util_send ("(");
   for (i = 1; i <= count; i++)
     {
-      char buf[128];
+      const char *str;
+      int is_group = 0;
 
       util_send ("(");
 
-      *buf = '\0';
-      mu_address_get_personal (address, i, buf, sizeof (buf), NULL);
-      util_send_qstring (buf);
+      mu_address_sget_personal (address, i, &str);
+      util_send_qstring (str);
+      util_send (" ");
+
+      mu_address_sget_route (address, i, &str);
+      util_send_qstring (str);
 
       util_send (" ");
 
-      *buf = '\0';
-      mu_address_get_route (address, i, buf, sizeof (buf), NULL);
-      util_send_qstring (buf);
+      mu_address_is_group (address, i, &is_group);
+      str = NULL;
+      if (is_group)
+	mu_address_sget_personal (address, i, &str);
+      else
+	mu_address_sget_local_part (address, i, &str);
+
+      util_send_qstring (str);
 
       util_send (" ");
 
-      *buf = '\0';
-      {
-	int is_group = 0;
-
-	mu_address_is_group (address, i, &is_group);
-	if (is_group)
-	  mu_address_get_personal (address, i, buf, sizeof (buf), NULL);
-	else
-	  mu_address_get_local_part (address, i, buf, sizeof (buf), NULL);
-      }
-      util_send_qstring (buf);
-
-      util_send (" ");
-
-      *buf = '\0';
-      mu_address_get_domain (address, i, buf, sizeof (buf), NULL);
-      util_send_qstring (buf);
+      mu_address_sget_domain (address, i, &str);
+      util_send_qstring (str);
 
       util_send (")");
     }
