@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
    Copyright (C) 1999, 2001, 2002, 2003, 2004, 
-   2005, 2006, 2007 Free Software Foundation, Inc.
+   2005, 2006, 2007, 2008 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -26,7 +26,7 @@ static int add2set (size_t **, int *, unsigned long);
 static const char *sc2string (int);
 
 /* Get the next space/CR/NL separated word, some words are between double
-   quotes, strtok() can not handle it.  */
+   quotes, strtok() cannot handle it.  */
 char *
 util_getword (char *s, char **save)
 {
@@ -360,8 +360,8 @@ util_send (const char *format, ...)
   if (!buf)
       imap4d_bye (ERR_NO_MEM);
 
-  if (daemon_param.transcript)
-    syslog (LOG_DEBUG, "sent: %s", buf);
+  if (imap4d_transcript)
+    mu_diag_output (MU_DIAG_DEBUG, "sent: %s", buf);
 
   status = mu_stream_sequential_write (ostream, buf, strlen (buf));
   free (buf);
@@ -414,8 +414,8 @@ util_out (int rc, const char *format, ...)
   if (!buf)
     imap4d_bye (ERR_NO_MEM);
 
-  if (daemon_param.transcript)
-    syslog (LOG_DEBUG, "sent: %s", buf);
+  if (imap4d_transcript)
+    mu_diag_output (MU_DIAG_DEBUG, "sent: %s", buf);
 
   status = mu_stream_sequential_write (ostream, buf, strlen (buf));
   free (buf);
@@ -455,8 +455,8 @@ util_finish (struct imap4d_command *command, int rc, const char *format, ...)
   strcat (buf, tempbuf);
   free (tempbuf);
 
-  if (daemon_param.transcript)
-    syslog (LOG_DEBUG, "sent: %s\r\n", buf);
+  if (imap4d_transcript)
+    mu_diag_output (MU_DIAG_DEBUG, "sent: %s\r\n", buf);
 
   mu_stream_sequential_write (ostream, buf, strlen (buf));
   free (buf);
@@ -503,11 +503,11 @@ imap4d_readline (void)
       size_t sz;
       int rc;
       
-      alarm (daemon_param.timeout);
+      alarm (idle_timeout);
       rc = mu_stream_sequential_readline (istream, buffer, sizeof (buffer), &sz);
       if (sz == 0)
 	{
-	  syslog (LOG_INFO, _("Unexpected eof on input"));
+	  mu_diag_output (MU_DIAG_INFO, _("Unexpected eof on input"));
 	  imap4d_bye (ERR_NO_OFILE);
 	}
       else if (rc)
@@ -516,7 +516,7 @@ imap4d_readline (void)
 	  if (mu_stream_strerror (istream, &p))
 	    p = strerror (errno);
 
-	  syslog (LOG_INFO, _("Error reading from input file: %s"), p);
+	  mu_diag_output (MU_DIAG_INFO, _("Error reading from input file: %s"), p);
 	  imap4d_bye (ERR_NO_OFILE);
 	}
       alarm (0);
@@ -577,8 +577,8 @@ imap4d_readline (void)
 	}
     }
   while (number > 0 || (total && line[total - 1] != '\n'));
-  if (daemon_param.transcript)
-    syslog (LOG_DEBUG, "recv: %s", line);
+  if (imap4d_transcript)
+    mu_diag_output (MU_DIAG_DEBUG, "recv: %s", line);
   return line;
 }
 
@@ -676,6 +676,9 @@ sc2string (int rc)
 
     case RESP_BYE:
       return "BYE ";
+
+    case RESP_PREAUTH:
+      return "PREAUTH ";
     }
   return "";
 }
@@ -727,7 +730,7 @@ util_parse_internal_date (char *date, time_t * timep)
 
 
 int
-util_parse_822_date (char *date, time_t * timep)
+util_parse_822_date (const char *date, time_t * timep)
 {
   struct tm tm;
   mu_timezone tz;
@@ -1003,8 +1006,8 @@ util_localname ()
 	}
       if (status)
 	{
-	  syslog (LOG_CRIT, _("Cannot find out my own hostname"));
-	  exit (1);
+	  mu_diag_output (MU_DIAG_CRIT, _("Cannot find out my own hostname"));
+	  exit (EX_OSERR);
 	}
 
       hp = gethostbyname (name);
@@ -1025,53 +1028,61 @@ util_localname ()
   return localname;
 }
 
-/* Match STRING against the IMAP4 wildard pattern PATTERN */
+/* Match STRING against the IMAP4 wildcard pattern PATTERN. */
+
+#define WILD_FALSE 0
+#define WILD_TRUE  1
+#define WILD_ABORT 2
 
 int
-util_wcard_match (const char *string, const char *pattern, const char *delim)
+_wild_match (const char *expr, const char *name, char delim)
 {
-  const char *p = pattern, *n = string;
-  char c;
-
-  for (; (c = *p++) != '\0' && *n; n++)
+  while (expr && *expr)
     {
-      switch (c)
+      if (*name == 0 && *expr != '*')
+	return WILD_ABORT;
+      switch (*expr)
 	{
-	case '%':
-	  if (*p == '\0')
-	    {
-	      /* Matches everything except '/' */
-	      for (; *n && *n != delim[0]; n++)
-		;
-	      return (*n == delim[0]) ? WCARD_RECURSE_MATCH : WCARD_MATCH;
-	    }
-	  else
-	    for (; *n != '\0'; ++n)
-	      if (util_wcard_match (n, p, delim) == WCARD_MATCH)
-		return WCARD_MATCH;
-	  break;
-
 	case '*':
-	  if (*p == '\0')
-	    return WCARD_RECURSE_MATCH;
-	  for (; *n != '\0'; ++n)
+	  while (*++expr == '*')
+	    ;
+	  if (*expr == 0)
+	    return WILD_TRUE;
+	  while (*name)
 	    {
-	      int status = util_wcard_match (n, p, delim);
-	      if (status == WCARD_MATCH || status == WCARD_RECURSE_MATCH)
-		return status;
+	      int res = _wild_match (expr, name++, delim);
+	      if (res != WILD_FALSE)
+		return res;
 	    }
-	  break;
+	  return WILD_ABORT;
 
+	case '%':
+	  while (*++expr == '%')
+	    ;
+	  if (*expr == 0)
+	    return strchr (name, delim) ? WILD_FALSE : WILD_TRUE;
+	  while (*name && *name != delim)
+	    {
+	      int res = _wild_match (expr, name++, delim);
+	      if (res != WILD_FALSE)
+		return res;
+	    }
+	  return _wild_match (expr, name, delim);
+	  
 	default:
-	  if (c != *n)
-	    return WCARD_NOMATCH;
+	  if (*expr != *name)
+	    return WILD_FALSE;
+	  expr++;
+	  name++;
 	}
     }
+  return *name == 0;
+}
 
-  if (!c && !*n)
-    return WCARD_MATCH;
-
-  return WCARD_NOMATCH;
+int
+util_wcard_match (const char *name, const char *expr, const char *delim)
+{
+  return _wild_match (expr, name, delim[0]) != WILD_TRUE;
 }
 
 /* Return the uindvalidity of a mailbox.
@@ -1150,7 +1161,7 @@ util_wait_input (int timeout)
   status = mu_stream_wait (istream, &wflags, &tv);
   if (status)
     {
-      syslog (LOG_ERR, _("Cannot poll input stream: %s"),
+      mu_diag_output (MU_DIAG_ERROR, _("Cannot poll input stream: %s"),
 	      mu_strerror(status));
       return -1;
     }
@@ -1184,7 +1195,7 @@ imap4d_init_tls_server ()
     {
       const char *p;
       mu_stream_strerror (stream, &p);
-      syslog (LOG_ERR, _("Cannot open TLS stream: %s"), p);
+      mu_diag_output (MU_DIAG_ERROR, _("Cannot open TLS stream: %s"), p);
       return 0;
     }
 
