@@ -1,9 +1,10 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2002, 2006 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2000, 2001, 2002, 2006,
+   2007 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
+   the Free Software Foundation; either version 3, or (at your option)
    any later version.
 
    GNU Mailutils is distributed in the hope that it will be useful,
@@ -39,6 +40,8 @@ mu_fcheck_perm (int fd, int mode)
 {
   struct stat st;
 
+  if (mode == 0)
+    return 0;
   if (fstat (fd, &st) == -1)
     {
       if (errno == ENOENT)
@@ -76,24 +79,6 @@ mu_check_perm (const char *name, int mode)
   return 0;
 }
 
-static char *
-strip_suffix (const char *name, const char *suffix)
-{
-  int nlen = strlen (name);
-  int slen = strlen (suffix);
-  char *p;
-  
-  if (nlen > slen && strcmp (name + nlen - slen, suffix) == 0)
-    {
-      p = xmalloc (nlen - slen + 1);
-      memcpy (p, name, nlen - slen);
-      p[nlen - slen] = 0;
-    }
-  else
-    p = xstrdup (name);
-  return p;
-}
-
 static char *
 make_db_name (const char *name, const char *suffix)
 {
@@ -142,12 +127,15 @@ mu_dbm_open (char *name, DBM_FILE *db, int flags, int mode)
     case MU_STREAM_CREAT:
       f = GDBM_NEWDB;
       break;
+      
     case MU_STREAM_READ:
       f = GDBM_READER;
       break;
+      
     case MU_STREAM_RDWR:
       f = GDBM_WRCREAT;
       break;
+      
     default:
       free (pfname);
       errno = EINVAL;
@@ -197,6 +185,15 @@ mu_dbm_nextkey (DBM_FILE db, DBM_DATUM key)
   return gdbm_nextkey (db, key);
 }
 
+void
+mu_dbm_datum_free (DBM_DATUM *datum)
+{
+  void *ptr = MU_DATUM_PTR (*datum);
+  if (ptr)
+    free (ptr);
+  MU_DATUM_PTR (*datum) = 0;
+}
+
 #elif defined(WITH_BDB)
 
 #define DB_SUFFIX ".db"
@@ -221,6 +218,7 @@ mu_dbm_open (char *name, DBM_FILE *dbm, int flags, int mode)
   if (mu_check_perm (pfname, mode))
     {
       free (pfname);
+      errno = MU_ERR_UNSAFE_PERMS;
       return -1;
     }
 
@@ -229,12 +227,15 @@ mu_dbm_open (char *name, DBM_FILE *dbm, int flags, int mode)
     case MU_STREAM_CREAT:
       f = DB_CREATE|DB_TRUNCATE;
       break;
+      
     case MU_STREAM_READ:
       f = DB_RDONLY;
       break;
+      
     case MU_STREAM_RDWR:
       f = DB_CREATE;
       break;
+      
     default:
       free (pfname);
       errno = EINVAL;
@@ -348,6 +349,12 @@ mu_dbm_nextkey (DBM_FILE db, DBM_DATUM pkey /*unused*/)
   return key;
 }
 
+void
+mu_dbm_datum_free (DBM_DATUM *datum)
+{
+  /* empty */
+}
+
 #elif defined(WITH_NDBM)
 
 #define DB_SUFFIX ".pag"
@@ -373,12 +380,15 @@ mu_dbm_open (char *name, DBM_FILE *db, int flags, int mode)
     case MU_STREAM_CREAT:
       f = O_CREAT|O_TRUNC|O_RDWR;
       break;
+      
     case MU_STREAM_READ:
       f = O_RDONLY;
       break;
+      
     case MU_STREAM_RDWR:
       f = O_CREAT|O_RDWR;
       break;
+      
     default:
       errno = EINVAL;
       return -1;
@@ -437,111 +447,10 @@ mu_dbm_nextkey (DBM_FILE db, DBM_DATUM key)
   return dbm_nextkey (db, key);
 }
 
-#elif defined(WITH_OLD_DBM)
-
-#define DB_SUFFIX ".pag"
-
-int
-mu_dbm_stat (char *name, struct stat *sb)
+void
+mu_dbm_datum_free (DBM_DATUM *datum)
 {
-  int rc;
-  char *pfname = make_db_name (name, DB_SUFFIX);
-  rc = stat (pfname, sb);
-  free (pfname);
-  return rc;
+  /* empty */
 }
-
-/*ARGSUSED*/
-int
-mu_dbm_open (char *name, DBM_FILE *db, int flags, int mode)
-{
-  int f;
-  char *pfname;
-  
-  switch (flags)
-    {
-    case MU_STREAM_CREAT:
-      f = O_CREAT|O_TRUNC|O_RDWR;
-      break;
-    case MU_STREAM_READ:
-      f = O_RDONLY;
-      break;
-    case MU_STREAM_RDWR:
-      f = O_CREAT|O_RDWR;
-      break;
-    default:
-      return -1;
-    }
-  pfname = strip_suffix (name, DB_SUFFIX);
-  if (f & O_CREAT)
-    {
-      char *p;
-      int fd;
-
-      p = make_db_name (pfname, ".pag");
-      fd = open (p, f, mode);
-      free (p);
-      if (fd < 0)
-	{
-	  free (pfname);
-	  return -1;
-	}
-      close(fd);
-
-      p = make_db_name (pfname, ".dir");
-      fd = open (p, f, mode);
-      free (p);
-      if (fd < 0)
-	{
-	  free (pfname);
-	  return -1;
-	}
-      close (fd);
-    }
-  rc = dbminit (pfname);
-  free (pfname);
-  return rc;
-}
-
-/*ARGSUSED*/
-int
-mu_dbm_close (DBM_FILE db)
-{
-  dbmclose ();
-  return 0;
-}
-
-/*ARGSUSED*/
-int
-mu_dbm_fetch (DBM_FILE db, DBM_DATUM key, DBM_DATUM *ret)
-{
-  *ret = fetch (key);
-  return ret->dptr == NULL;
-}
-
-int
-mu_dbm_delete (DBM_FILE db, DBM_DATUM key)
-{
-  return delete (key);
-}
-
-int
-mu_dbm_insert (DBM_FILE db, DBM_DATUM key, DBM_DATUM contents, int replace)
-{
-  return store (key, contents);
-}
-
-DBM_DATUM
-mu_dbm_firstkey (DBM_FILE db)
-{
-  return firstkey ();
-}
-
-DBM_DATUM
-mu_dbm_nextkey (DBM_FILE db, DBM_DATUM key)
-{
-  return nextkey (key);
-}
-
 #endif
 
