@@ -1,9 +1,9 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2003, 2004, 2005 Free Software Foundation, Inc.
+   Copyright (C) 2003, 2004, 2005, 2007, 2008 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
+   the Free Software Foundation; either version 3, or (at your option)
    any later version.
 
    GNU Mailutils is distributed in the hope that it will be useful,
@@ -32,6 +32,7 @@
 #include <signal.h>
 #include <mailutils/libsieve.h>
 #include <mailutils/mu_auth.h>
+#include <mailutils/nls.h>
 
 #define DEFAULT_SPAMD_PORT 783
 
@@ -42,7 +43,7 @@ static int
 spamd_connect_tcp (mu_sieve_machine_t mach, mu_stream_t *stream,
 		   char *host, int port)
 {
-  int rc = mu_tcp_stream_create (stream, host, port, 0);
+  int rc = mu_tcp_stream_create (stream, host, port, MU_STREAM_NO_CHECK);
   if (rc)
     {
       mu_sieve_error (mach, "mu_tcp_stream_create: %s", mu_strerror (rc));
@@ -50,50 +51,29 @@ spamd_connect_tcp (mu_sieve_machine_t mach, mu_stream_t *stream,
     }
   rc = mu_stream_open (*stream);
   if (rc)
-    mu_sieve_error (mach, "opening tcp stream: %s", mu_strerror (rc));
+    {
+      mu_sieve_error (mach, "opening tcp stream: %s", mu_strerror (rc));
+      mu_stream_destroy (stream, NULL);
+    }
   return rc;
 }
 
 static int
 spamd_connect_socket (mu_sieve_machine_t mach, mu_stream_t *stream, char *path)
 {
-  /* FIXME: A library deficiency: we cannot create a unix socket stream */
-  int fd, rc;
-  FILE *fp;
-  struct sockaddr_un addr;
-  
-  if ((fd = socket (PF_UNIX, SOCK_STREAM, 0)) < 0)
-    {
-      mu_sieve_error (mach, "socket: %s", mu_strerror (errno));
-      return errno;
-    }
-
-  memset (&addr, 0, sizeof addr);
-  addr.sun_family = AF_UNIX;
-  strncpy (addr.sun_path, path, sizeof addr.sun_path - 1);
-  addr.sun_path[sizeof addr.sun_path - 1] = 0;
-  if (connect (fd, (struct sockaddr *) &addr, sizeof(addr)))
-    {
-      mu_sieve_error (mach, "connect: %s", mu_strerror (errno));
-      close (fd);
-      return errno;
-    }
-
-  fp = fdopen (fd, "w+");
-  rc = mu_stdio_stream_create (stream, fp, MU_STREAM_RDWR);
+  int rc = mu_socket_stream_create (stream, path, MU_STREAM_NO_CHECK);
   if (rc)
     {
-      mu_sieve_error (mach, "mu_stdio_stream_create: %s", mu_strerror (rc));
-      fclose (fp);
+      mu_sieve_error (mach, "mu_socket_stream_create: %s", mu_strerror (rc));
       return rc;
     }
-
   rc = mu_stream_open (*stream);
   if (rc)
     {
-      mu_sieve_error (mach, "mu_stream_open: %s", mu_strerror (rc));
-      mu_stream_destroy (stream, mu_stream_get_owner (*stream));
+      mu_sieve_error (mach, "opening socket stream: %s", mu_strerror (rc));
+      mu_stream_destroy (stream, NULL);
     }
+
   return rc;
 }
 
@@ -104,15 +84,6 @@ spamd_destroy (mu_stream_t *stream)
   mu_stream_destroy (stream, mu_stream_get_owner (*stream));
 }
 
-static void
-spamd_shutdown (mu_stream_t stream, int flag)
-{
-  mu_transport_t trans;
-  mu_stream_flush (stream);
-  mu_stream_get_transport (stream, &trans);
-  shutdown (fileno ((FILE*)trans), flag);
-}
-
 static void
 spamd_send_command (mu_stream_t stream, const char *fmt, ...)
 {
@@ -253,7 +224,7 @@ spamd_abort (mu_sieve_machine_t mach, mu_stream_t *stream, signal_handler handle
 static int got_sigpipe;
 
 static RETSIGTYPE
-sigpipe_handler (int sig ARG_UNUSED)
+sigpipe_handler (int sig MU_ARG_UNUSED)
 {
   got_sigpipe = 1;
 }
@@ -343,26 +314,26 @@ spamd_test (mu_sieve_machine_t mach, mu_list_t args, mu_list_t tags)
   
   spamd_send_command (stream, "");
   spamd_send_message (stream, msg);
-  spamd_shutdown (stream, SHUT_WR);
+  mu_stream_shutdown (stream, MU_STREAM_WRITE);
 
   spamd_read_line (mach, stream, buffer, sizeof buffer, NULL);
 
   if (got_sigpipe)
     {
-      mu_sieve_error (mach, "remote side has closed connection");
+      mu_sieve_error (mach, _("remote side has closed connection"));
       spamd_abort (mach, &stream, handler);
     }
 
   if (sscanf (buffer, "SPAMD/%18s %d %*s", version_str, &response) != 2)
     {
-      mu_sieve_error (mach, "spamd responded with bad string '%s'", buffer);
+      mu_sieve_error (mach, _("spamd responded with bad string '%s'"), buffer);
       spamd_abort (mach, &stream, handler);
     }
   
   decode_float (&version, version_str, 1);
   if (version < 10)
     {
-      mu_sieve_error (mach, "unsupported SPAMD version: %s", version_str);
+      mu_sieve_error (mach, _("unsupported SPAMD version: %s"), version_str);
       spamd_abort (mach, &stream, handler);
     }
 
@@ -375,7 +346,8 @@ spamd_test (mu_sieve_machine_t mach, mu_list_t args, mu_list_t tags)
   if (sscanf (buffer, "Spam: %5s ; %20s / %20s",
 	      spam_str, score_str, threshold_str) != 3)
     {
-      mu_sieve_error (mach, "spamd responded with bad Spam header '%s'", buffer);
+      mu_sieve_error (mach, _("spamd responded with bad Spam header '%s'"), 
+                      buffer);
       spamd_abort (mach, &stream, handler);
     }
 
@@ -406,14 +378,15 @@ spamd_test (mu_sieve_machine_t mach, mu_list_t args, mu_list_t tags)
   rc = mu_message_get_header (msg, &hdr);
   if (rc)
     {
-      mu_sieve_error (mach, "cannot get message header: %s", mu_strerror (rc));
+      mu_sieve_error (mach, _("cannot get message header: %s"), 
+                      mu_strerror (rc));
       spamd_abort (mach, &stream, handler);
     }
 
-  mu_header_set_value (hdr, "X-Spamd-Status", spam_str, 1);
-  mu_header_set_value (hdr, "X-Spamd-Score", score_str, 1);
-  mu_header_set_value (hdr, "X-Spamd-Threshold", threshold_str, 1);
-  mu_header_set_value (hdr, "X-Spamd-Keywords", buffer, 1);
+  mu_header_append (hdr, "X-Spamd-Status", spam_str);
+  mu_header_append (hdr, "X-Spamd-Score", score_str);
+  mu_header_append (hdr, "X-Spamd-Threshold", threshold_str);
+  mu_header_append (hdr, "X-Spamd-Keywords", buffer);
 
   while (spamd_read_line (mach, stream, buffer, sizeof buffer, &size) == 0
 	 && size > 0)
