@@ -1,10 +1,11 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2002, 2005 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2000, 2001, 2002, 2005,
+   2007, 2008 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
+   version 3 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,22 +28,46 @@
 #include <string.h>
 #include <sieve.h>
 #include <ltdl.h>
+#include <ctype.h>
 
 typedef int (*sieve_module_init_t) (mu_sieve_machine_t mach);
 
+#if 0
+/* FIXME: See comment below */ 
 static void
 _free_loaded_module (void *data)
 {
   lt_dlclose ((lt_dlhandle)data);
   lt_dlexit ();
 }
+#endif
 
+static int _add_load_dir (void *, void *);
+
+static int
+sieve_init_load_path ()
+{
+  static int inited = 0;
+
+  if (!inited)
+    {
+      if (lt_dlinit ())
+	return 1;
+#ifdef SIEVE_MODDIR
+      _add_load_dir (SIEVE_MODDIR, NULL);
+      inited = 1;
+#endif
+    }
+  return 0;
+}
+  
+     
 static lt_dlhandle
 load_module (mu_sieve_machine_t mach, const char *name)
 {
   lt_dlhandle handle;
 
-  if (lt_dlinit ())
+  if (sieve_init_load_path ())
     return NULL;
 
   handle = lt_dlopenext (name);
@@ -53,7 +78,13 @@ load_module (mu_sieve_machine_t mach, const char *name)
       if (init)
 	{
 	  init (mach);
-	  mu_sieve_machine_add_destructor (mach, _free_loaded_module, handle);
+	  /* FIXME: We used to have this:
+  	       mu_sieve_machine_add_destructor (mach, _free_loaded_module,
+ 	                                        handle);
+             However, unloading modules can lead to random segfaults in
+	     case they allocated any global-access data (e.g. mach->msg).
+	     In particular, this was the case with extensions/pipe.c. 
+	  */
 	  return handle;
 	}
       else
@@ -65,7 +96,7 @@ load_module (mu_sieve_machine_t mach, const char *name)
 
   if (!handle)
     {
-      mu_sieve_error (mach, "%s", lt_dlerror ());
+      mu_sieve_error (mach, "%s: %s", name, lt_dlerror ());
       lt_dlexit ();
     }
   return handle;
@@ -98,7 +129,7 @@ mu_sieve_load_ext (mu_sieve_machine_t mach, const char *name)
 }
 
 static int
-_load_dir (void *item, void *unused)
+_add_load_dir (void *item, void *unused)
 {
   return lt_dladdsearchdir (item);
 }
@@ -106,15 +137,15 @@ _load_dir (void *item, void *unused)
 int
 sieve_load_add_path (mu_list_t path)
 {
-  if (lt_dlinit ())
+  if (sieve_init_load_path ())
     return 1;
-  return mu_list_do (path, _load_dir, NULL);
+  return mu_list_do (path, _add_load_dir, NULL);
 }
 
 int
 sieve_load_add_dir (mu_sieve_machine_t mach, const char *name)
 {
-  if (lt_dlinit ())
+  if (sieve_init_load_path ())
     return 1;
   mu_sieve_machine_add_destructor (mach, (mu_sieve_destructor_t) lt_dlexit, NULL);
   return lt_dladdsearchdir (name);
