@@ -1,10 +1,11 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2002, 2005 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2000, 2001, 2002, 2005,
+   2007 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
+   version 3 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -286,7 +287,7 @@ sieve_run (mu_sieve_machine_t mach)
     mu_sieve_log_action (mach, "IMPLICIT KEEP", NULL);
   
   if (INSTR_DEBUG (mach))
-    mu_sieve_debug (mach, "%4lu: STOP\n", mach->pc);
+    mu_sieve_debug (mach, "%4lu: STOP\n", (unsigned long) mach->pc);
   
   return 0;
 }
@@ -304,7 +305,7 @@ mu_sieve_disass (mu_sieve_machine_t mach)
 }
   
 static int
-_sieve_action (mu_observer_t obs, size_t type)
+_sieve_action (mu_observer_t obs, size_t type, void *data, void *action_data)
 {
   mu_sieve_machine_t mach;
   
