@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
    Copyright (C) 1999, 2000, 2001, 2002, 2005, 
-   2007 Free Software Foundation, Inc.
+   2007, 2008 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -18,6 +18,7 @@
    MA 02110-1301 USA */
 
 #include <mail.local.h>
+#include "mailutils/libargp.h"
 
 int multiple_delivery;     /* Don't return errors when delivering to multiple
 			      recipients */
@@ -46,7 +47,6 @@ void close_fds (void);
 int make_tmp (const char *from, mu_mailbox_t *mbx);
 void deliver (mu_mailbox_t msg, char *name);
 void guess_retval (int ec);
-void mailer_err (char *fmt, ...);
 void notify_biff (mu_mailbox_t mbox, char *name, size_t size);
 
 const char *program_version = "mail.local (" PACKAGE_STRING ")";
@@ -55,7 +55,7 @@ N_("GNU mail.local -- the local MDA")
 "\v"
 N_("Debug flags are:\n\
   g - guimb stack traces\n\
-  T - mailutils traces (MU_DEBUG_TRACE)\n\
+  T - mailutils traces (MU_DEBUG_TRACE0-MU_DEBUG_TRACE9)\n\
   P - network protocols (MU_DEBUG_PROT)\n\
   t - sieve trace (MU_SIEVE_DEBUG_TRACE)\n\
   i - sieve instructions trace (MU_SIEVE_DEBUG_INSTR)\n\
@@ -113,11 +113,12 @@ static struct argp argp = {
 static const char *argp_capa[] = {
   "auth",
   "common",
+  "debug",
   "license",
   "logging",
   "mailbox",
+  "locking",
   "mailer",
-  "sieve",
   NULL
 };
 
@@ -128,25 +129,74 @@ char *saved_envelope;  /* A hack to spare mu_envelope_ calls */
 
 #define D_DEFAULT "9s"
 
+static void
+set_debug_flags (mu_debug_t debug, const char *arg)
+{
+  for (; *arg; arg++)
+    {
+      switch (*arg)
+	{
+	case 'g':
+#ifdef WITH_GUILE
+	  debug_guile = 1;
+#endif
+	  break;
+
+	case 't':
+	  sieve_debug_flags |= MU_SIEVE_DEBUG_TRACE;
+	  break;
+	  
+	case 'i':
+	  sieve_debug_flags |= MU_SIEVE_DEBUG_INSTR;
+	  break;
+	  
+	case 'l':
+	  sieve_enable_log = 1;
+	  break;
+	  
+	case 'T':
+	  debug_flags |= MU_DEBUG_LEVEL_UPTO (MU_DEBUG_TRACE7);
+	  break;
+	  
+	case 'P':
+	  debug_flags |= MU_DEBUG_LEVEL_MASK (MU_DEBUG_PROT);
+	  break;
+
+	default:
+	  if (isdigit (*arg))
+	    debug_level = *arg - '0';
+	  else if (debug)
+	    mu_cfg_format_error (debug, MU_DEBUG_ERROR,
+				 _("%c is not a valid debug flag"),
+				 *arg);
+	  else
+	    mu_error (_("%c is not a valid debug flag"), *arg);
+	  break;
+	}
+    }
+}
+
 static error_t
 parse_opt (int key, char *arg, struct argp_state *state)
 {
+  static struct mu_argp_node_list lst;
+
   switch (key)
     {
     case ARG_MULTIPLE_DELIVERY:
-      multiple_delivery = 1;
+      mu_argp_node_list_new (&lst, "ex-multiple-delivery-success", "yes");
       break;
 
     case ARG_QUOTA_TEMPFAIL:
-      ex_quota_tempfail = 1;
+      mu_argp_node_list_new (&lst, "ex-quota-tempfail", "yes");
       break;
 
     case ARG_MESSAGE_ID_HEADER:
-      message_id_header = arg;
+      mu_argp_node_list_new (&lst, "message-id-header", arg);
       break;
 
     case ARG_QUOTA_QUERY:
-      quota_query = arg;
+      mu_argp_node_list_new (&lst, "quota-query", arg);
       break;
       
     case 'r':
@@ -161,62 +211,31 @@ parse_opt (int key, char *arg, struct argp_state *state)
       
 #ifdef USE_DBM
     case 'q':
-      quotadbname = arg;
+      mu_argp_node_list_new (&lst, "quota-db", arg);
       break;
 #endif
 
 #ifdef WITH_GUILE	
     case 's':
-      progfile_pattern = arg;
+      mu_argp_node_list_new (&lst, "source", arg);
       break;
 #endif
 
     case 'S':
-      sieve_pattern = arg;
+      mu_argp_node_list_new (&lst, "sieve-filter", arg);
+      
       break;
       
     case 'x':
-      do
-	{
-	  if (!arg)
-	    arg = D_DEFAULT;
-	  switch (*arg)
-	    {
-	    case 'g':
-#ifdef WITH_GUILE
-	      debug_guile = 1;
-#endif
-	      break;
-
-	    case 't':
-	      sieve_debug_flags |= MU_SIEVE_DEBUG_TRACE;
-	      break;
-
-	    case 'i':
-	      sieve_debug_flags |= MU_SIEVE_DEBUG_INSTR;
-	      break;
-	      
-	    case 'l':
-	      sieve_enable_log = 1;
-	      break;
-	      
-	    case 'T':
-	      debug_flags |= MU_DEBUG_TRACE;
-	      break;
-
-	    case 'P':
-	      debug_flags |= MU_DEBUG_PROT;
-	      break;
-
-	    default:
-	      if (isdigit (*arg))
-		debug_level = *arg - '0';
-	      else
-		argp_error (state, _("%c is not a valid debug flag"), *arg);
-	      break;
-	    }
-	}
-      while (*++arg);
+      mu_argp_node_list_new (&lst, "debug", arg ? arg : D_DEFAULT);
+      break;
+      
+    case ARGP_KEY_INIT:
+      mu_argp_node_list_init (&lst);
+      break;
+      
+    case ARGP_KEY_FINI:
+      mu_argp_node_list_finish (&lst, NULL, NULL);
       break;
       
     default:
@@ -228,29 +247,76 @@ parse_opt (int key, char *arg, struct argp_state *state)
   return 0;
 }
 
-
+
 static int
-_mu_debug_printer (mu_debug_t unused, size_t level, const char *fmt,
-		   va_list ap)
+cb_debug (mu_debug_t debug, void *data, char *arg)
 {
-  vsyslog ((level == MU_DEBUG_ERROR) ? LOG_ERR : LOG_DEBUG, fmt, ap);
+  set_debug_flags (debug, arg);
   return 0;
 }
 
+struct mu_cfg_param mail_local_cfg_param[] = {
+  { "ex-multiple-delivery-success", mu_cfg_bool, &multiple_delivery, 0, NULL,
+    N_("In case of multiple delivery, exit with code 0 if at least one "
+       "delivery succeeded.") },
+  { "ex-quota-tempfail", mu_cfg_bool, &ex_quota_tempfail, 0, NULL,
+    N_("Indicate temporary failure if the recipient is over his mail quota.")
+  },
+#ifdef USE_DBM
+  { "quota-db", mu_cfg_string, &quotadbname, 0, NULL,
+    N_("Name of DBM quota database file."),
+    N_("file") },
+#endif
+#ifdef USE_SQL
+  { "quota-query", mu_cfg_string, &quota_query, 0, NULL,
+    N_("SQL query to retrieve mailbox quota.  This is deprecated, use "
+       "sql { ... } instead."),
+    N_("query") },
+#endif
+  { "sieve-filter", mu_cfg_string, &sieve_pattern, 0, NULL,
+    N_("File name or name pattern for Sieve filter file."),
+    N_("file-or-pattern") },
+  { "message-id-header", mu_cfg_string, &message_id_header, 0, NULL,
+    N_("When logging Sieve actions, identify messages by the value of "
+       "this header."),
+    N_("name") },
+#ifdef WITH_GUILE
+  { "guile-filter", mu_cfg_string, &progfile_pattern, 0, NULL,
+    N_("File name or name pattern for Guile filter file."),
+    N_("file-or-pattern") },
+#endif
+  { "debug", mu_cfg_callback, NULL, 0, cb_debug,
+    N_("Set mail.local debug level.  Debug level consists of one or more "
+       "of the following letters:\n"
+       "  g - guimb stack traces\n"
+       "  t - sieve trace (MU_SIEVE_DEBUG_TRACE)\n"
+       "  i - sieve instructions trace (MU_SIEVE_DEBUG_INSTR)\n"
+       "  l - sieve action logs\n") },
+  { NULL }
+};
+
+
+
 static int
 _sieve_debug_printer (void *unused, const char *fmt, va_list ap)
 {
-  vsyslog (LOG_DEBUG, fmt, ap);
+  mu_diag_vprintf (MU_DIAG_DEBUG, fmt, ap);
   return 0;
 }
 
 static void
 _sieve_action_log (void *user_name,
-		   const mu_sieve_locus_t *locus, size_t msgno, mu_message_t msg,
+		   const mu_sieve_locus_t *locus, size_t msgno,
+		   mu_message_t msg,
 		   const char *action, const char *fmt, va_list ap)
 {
-  char *text = NULL;
+  int pfx = 0;
+  mu_debug_t debug;
+
+  mu_diag_get_debug (&debug);
+  mu_debug_set_locus (debug, locus->source_file, locus->source_line);
   
+  mu_diag_printf (MU_DIAG_NOTICE, _("(user %s) "), (char*) user_name);
   if (message_id_header)
     {
       mu_header_t hdr = NULL;
@@ -259,51 +325,42 @@ _sieve_action_log (void *user_name,
       if (mu_header_aget_value (hdr, message_id_header, &val) == 0
 	  || mu_header_aget_value (hdr, MU_HEADER_MESSAGE_ID, &val) == 0)
 	{
-	  asprintf (&text, _("%s:%lu: %s on msg %s"),
-		    locus->source_file,
-		    (unsigned long) locus->source_line,
-		    action, val);
+	  pfx = 1;
+	  mu_diag_printf (MU_DIAG_NOTICE, _("%s on msg %s"), action, val);
 	  free (val);
 	}
     }
-  if (text == NULL)
+  
+  if (!pfx)
     {
       size_t uid = 0;
       mu_message_get_uid (msg, &uid);
-      asprintf (&text, _("%s:%lu: %s on msg uid %d"),
-		locus->source_file,
-		(unsigned long)	locus->source_line,
-		action, uid);
+      mu_diag_printf (MU_DIAG_NOTICE, _("%s on msg uid %d"), action, uid);
     }
   
   if (fmt && strlen (fmt))
     {
-      char *diag = NULL;
-      vasprintf (&diag, fmt, ap);
-      syslog (LOG_NOTICE, _("(user %s) %s: %s"), (char*) user_name, text, diag);
-      free (diag);
+      mu_diag_printf (MU_DIAG_NOTICE, "; ");
+      mu_diag_vprintf (MU_DIAG_NOTICE, fmt, ap);
     }
-  else
-    syslog (LOG_NOTICE, _("(user %s) %s"), (char*) user_name, text);
-  free (text);
+  mu_diag_printf (MU_DIAG_NOTICE, "\n");
+  mu_debug_set_locus (debug, NULL, 0);
 }
 
 static int
 _sieve_parse_error (void *user_name, const char *filename, int lineno,
 		    const char *fmt, va_list ap)
 {
-  char *text;
-  vasprintf (&text, fmt, ap);
+  mu_debug_t debug;
+
+  mu_diag_get_debug (&debug);
   if (filename)
-    {
-      char *loc;
-      asprintf (&loc, "%s:%d: ", filename, lineno);
-      syslog (LOG_ERR, "%s: %s", loc, text);
-      free (loc);
-    }
-  else
-    syslog (LOG_ERR, _("(user %s) %s"), (char*)user_name, text);
-  free (text);
+    mu_debug_set_locus (debug, filename, lineno);
+
+  mu_diag_printf (MU_DIAG_NOTICE, _("(user %s) "), (char*) user_name);
+  mu_diag_vprintf (MU_DIAG_NOTICE, fmt, ap);
+  mu_diag_printf (MU_DIAG_NOTICE, "\n");
+  mu_debug_set_locus (debug, NULL, 0);
   return 0;
 }
 
@@ -319,29 +376,31 @@ main (int argc, char *argv[])
   umask (0077);
 
   /* Native Language Support */
-  mu_init_nls ();
+  MU_APP_INIT_NLS ();
 
   /* Default locker settings */
   mu_locker_set_default_flags (MU_LOCKER_PID|MU_LOCKER_RETRY,
-			    mu_locker_assign);
+			       mu_locker_assign);
   mu_locker_set_default_retry_timeout (1);
   mu_locker_set_default_retry_count (300);
 
-  /* Default error code for command line errors */
-  mu_argp_error_code = EX_CONFIG;
   /* Register needed modules */
   MU_AUTH_REGISTER_ALL_MODULES();
-  mu_argp_init (program_version, NULL);
-  mu_sieve_argp_init ();
   /* Parse command line */
-  mu_argp_parse (&argp, &argc, &argv, 0, argp_capa, &arg_index, NULL);
+  mu_gocs_register ("sieve", mu_sieve_module_init);
+  mu_argp_init (program_version, NULL);
+  if (mu_app_init (&argp, argp_capa, mail_local_cfg_param,
+		   argc, argv, 0, &arg_index, NULL))
+    exit (EX_CONFIG);
 
   uid = getuid ();
 
   if (uid == 0)
     {
-      openlog ("mail.local", LOG_PID, log_facility);
-      mu_error_set_print (mu_syslog_error_printer);
+      mu_debug_t debug;
+      openlog (MU_LOG_TAG (), LOG_PID, mu_log_facility);
+      mu_diag_get_debug (&debug);
+      mu_debug_set_print (debug, mu_diag_syslog_printer, NULL);
     }
   
   if (debug_flags)
@@ -359,12 +418,6 @@ main (int argc, char *argv[])
 		    mu_strerror (rc));
 	  exit (EX_TEMPFAIL);
 	}
-      if ((rc = mu_debug_set_print (mudebug, _mu_debug_printer, NULL)))
-	{
-	  mu_error (_("mu_debug_set_print failed: %s\n"),
-		    mu_strerror (rc));
-	  exit (EX_TEMPFAIL);
-	}
     }
   
   argc -= arg_index;
@@ -436,7 +489,7 @@ sieve_test (struct mu_auth_data *auth, mu_mailbox_t mbx)
   if (access (progfile, R_OK))
     {
       if (debug_level > 2)
-	syslog (LOG_DEBUG, _("Access to %s failed: %m"), progfile);
+	mu_diag_output (MU_DIAG_DEBUG, _("Access to %s failed: %m"), progfile);
     }
   else
     {
@@ -450,7 +503,8 @@ sieve_test (struct mu_auth_data *auth, mu_mailbox_t mbx)
       else
 	{
 	  mu_sieve_set_debug (mach, _sieve_debug_printer);
-	  mu_sieve_set_debug_level (mach, mudebug, sieve_debug_flags);
+	  mu_sieve_set_debug_object (mach, mudebug);
+	  mu_sieve_set_debug_level (mach, sieve_debug_flags);
 	  mu_sieve_set_parse_error (mach, _sieve_parse_error);
 	  if (sieve_enable_log)
 	    mu_sieve_set_logger (mach, _sieve_action_log);
@@ -469,7 +523,7 @@ sieve_test (struct mu_auth_data *auth, mu_mailbox_t mbx)
 		  chdir (auth->dir);
 		
 		  rc = mu_sieve_message (mach, msg);
-		  if (rc == 0)
+ 		  if (rc == 0)
 		    rc = mu_attribute_is_deleted (attr) == 0;
 
 		  switch_user_id (auth, 0);
@@ -534,7 +588,8 @@ switch_user_id (struct mu_auth_data *auth, int user)
 # error "No way to reset user privileges?"
 #endif
   if (rc < 0)
-    mailer_err ("setreuid(0, %d): %s (r=%d, e=%d)",
+    mailer_err ("cannot set user priviledes", auth->name,
+		"setreuid(0, %d): %s (r=%d, e=%d)",
 		uid, strerror (errno), getuid (), geteuid ());
   return rc;
 }
@@ -565,13 +620,16 @@ make_tmp (const char *from, mu_mailbox_t *mbox)
   tempfile = mu_tempname (NULL);
   if ((status = mu_file_stream_create (&stream, tempfile, MU_STREAM_RDWR)))
     {
-      mailer_err (_("Unable to open temporary file: %s"), mu_strerror (status));
+      mailer_err ("Temporary failure", NULL,
+		  _("Unable to open temporary file: %s"),
+		  mu_strerror (status));
       exit (exit_code);
     }
 
   if ((status = mu_stream_open (stream)))
     {
-      mailer_err (_("unable to open temporary file: %s"), mu_strerror (status));
+      mailer_err ("Temporary failure", NULL,
+		  _("unable to open temporary file: %s"), mu_strerror (status));
       exit (exit_code);
     }
   
@@ -601,7 +659,8 @@ make_tmp (const char *from, mu_mailbox_t *mbox)
 		}
 	      else
 		{
-		  mailer_err (_("Cannot determine sender address"));
+		  mailer_err ("Cannot determine sender address", NULL,
+			      _("Cannot determine sender address"));
 		  exit (EX_UNAVAILABLE);
 		}
 	      if (auth)
@@ -625,7 +684,9 @@ make_tmp (const char *from, mu_mailbox_t *mbox)
       
       if (status)
 	{
-	  mailer_err (_("Error writing temporary file: %s"), mu_strerror (status));
+	  mailer_err ("Temporary failure", NULL,
+		      _("Error writing temporary file: %s"),
+		      mu_strerror (status));
 	  mu_stream_destroy (&stream, mu_stream_get_owner (stream));
 	  return status;
 	}
@@ -642,7 +703,8 @@ make_tmp (const char *from, mu_mailbox_t *mbox)
   if (status)
     {
       errno = status;
-      mailer_err (_("Error writing temporary file: %s"), mu_strerror (status));
+      mailer_err ("Error writing temporary file", NULL,
+		  _("Error writing temporary file: %s"), mu_strerror (status));
       mu_stream_destroy (&stream, mu_stream_get_owner (stream));
       return status;
     }
@@ -652,7 +714,9 @@ make_tmp (const char *from, mu_mailbox_t *mbox)
       || (status = mu_mailbox_open (*mbox, MU_STREAM_READ))
       || (status = mu_mailbox_set_stream (*mbox, stream)))
     {
-      mailer_err (_("Error opening temporary file: %s"), mu_strerror (status));
+      mailer_err ("Error opening temporary file", NULL,
+		  _("Error opening temporary file: %s"),
+		  mu_strerror (status));
       mu_stream_destroy (&stream, mu_stream_get_owner (stream));
       return status;
     }
@@ -661,7 +725,8 @@ make_tmp (const char *from, mu_mailbox_t *mbox)
   if (status)
     {
       errno = status;
-      mailer_err (_("Error creating temporary message: %s"),
+      mailer_err ("Error collecting message", NULL,
+		  _("Error creating temporary message: %s"),
 		  mu_strerror (status));
       mu_stream_destroy (&stream, mu_stream_get_owner (stream));
       return status;
@@ -687,7 +752,8 @@ deliver (mu_mailbox_t imbx, char *name)
   auth = mu_get_auth_by_name (name);
   if (!auth)
     {
-      mailer_err (_("%s: no such user"), name);
+      mailer_err ("no such user", name,
+		  _("%s: no such user"), name);
       exit_code = EX_UNAVAILABLE;
       return;
     }
@@ -703,7 +769,8 @@ deliver (mu_mailbox_t imbx, char *name)
 
   if ((status = mu_mailbox_get_message (imbx, 1, &msg)) != 0)
     {
-      mailer_err (_("Cannot get message from the temporary mailbox: %s"),
+      mailer_err ("cannot read message", name,
+		  _("Cannot get message from the temporary mailbox: %s"),
 		  mu_strerror (status));
       mu_auth_data_free (auth);
       return;
@@ -711,7 +778,8 @@ deliver (mu_mailbox_t imbx, char *name)
 
   if ((status = mu_message_get_stream (msg, &istream)) != 0)
     {
-      mailer_err (_("Cannot get input message stream: %s"),
+      mailer_err ("cannot read message", name,
+		  _("Cannot get input message stream: %s"),
 		  mu_strerror (status));
       mu_auth_data_free (auth);
       return;
@@ -719,7 +787,8 @@ deliver (mu_mailbox_t imbx, char *name)
   
   if ((status = mu_mailbox_create (&mbox, auth->mailbox)) != 0)
     {
-      mailer_err (_("Cannot open mailbox %s: %s"),
+      mailer_err ("cannot read message", name,
+		  _("Cannot open mailbox %s: %s"),
 		  auth->mailbox, mu_strerror (status));
       mu_auth_data_free (auth);
       return;
@@ -738,7 +807,8 @@ deliver (mu_mailbox_t imbx, char *name)
     return;
   if (status != 0)
     {
-      mailer_err (_("Cannot open mailbox %s: %s"), path, mu_strerror (status));
+      mailer_err ("cannot open mailbox", name,
+		  _("Cannot open mailbox %s: %s"), path, mu_strerror (status));
       mu_mailbox_destroy (&mbox);
       return;
     }
@@ -749,7 +819,8 @@ deliver (mu_mailbox_t imbx, char *name)
 
   if (status)
     {
-      mailer_err (_("Cannot lock mailbox `%s': %s"), path,
+      mailer_err ("cannot lock mailbox", name,
+		  _("Cannot lock mailbox `%s': %s"), path,
 		  mu_strerror (status));
       mu_mailbox_destroy (&mbox);
       exit_code = EX_TEMPFAIL;
@@ -758,7 +829,8 @@ deliver (mu_mailbox_t imbx, char *name)
 
   if ((status = mu_mailbox_get_stream (mbox, &ostream)))
     {
-      mailer_err (_("Cannot get stream for mailbox %s: %s"),
+      mailer_err ("cannot access mailbox", name,
+		  _("Cannot get stream for mailbox %s: %s"),
 		  path, mu_strerror (status));
       mu_mailbox_destroy (&mbox);
       return;
@@ -766,7 +838,8 @@ deliver (mu_mailbox_t imbx, char *name)
 
   if ((status = mu_stream_size (ostream, &size)))
     {
-      mailer_err (_("Cannot get stream size (mailbox %s): %s"),
+      mailer_err ("cannot access mailbox", name,
+		  _("Cannot get stream size (mailbox %s): %s"),
 		  path, mu_strerror (status));
       mu_mailbox_destroy (&mbox);
       return;
@@ -780,7 +853,8 @@ deliver (mu_mailbox_t imbx, char *name)
     switch (check_quota (auth, size, &n))
       {
       case MQUOTA_EXCEEDED:
-	mailer_err (_("%s: mailbox quota exceeded for this recipient"), name);
+	mailer_err ("mailbox quota exceeded for this recipient", name,
+		    _("%s: mailbox quota exceeded for this recipient"), name);
 	exit_code = EX_QUOTA();
 	failed++;
 	break;
@@ -791,14 +865,17 @@ deliver (mu_mailbox_t imbx, char *name)
       default:
 	if ((status = mu_stream_size (istream, &isize)))
 	  {
-	    mailer_err (_("Cannot get stream size (input message %s): %s"),
+	    mailer_err ("cannot access mailbox", name,
+			_("Cannot get stream size (input message %s): %s"),
 			path, mu_strerror (status));
 	    exit_code = EX_UNAVAILABLE;
 	    failed++;
 	  }
 	else if (isize > n)
 	  {
-	    mailer_err (_("%s: message would exceed maximum mailbox size for this recipient"),
+	    mailer_err ("message would exceed maximum mailbox size for this recipient",
+			name,
+			_("%s: message would exceed maximum mailbox size for this recipient"),
 			name);
 	    exit_code = EX_QUOTA();
 	    failed++;
@@ -855,10 +932,12 @@ deliver (mu_mailbox_t imbx, char *name)
 	     original size */
 	  int rc = mu_stream_truncate (ostream, size);
 	  if (rc)
-	    mailer_err (_("Error writing to mailbox: %s. Mailbox NOT truncated: %s"),
+	    mailer_err ("write error", name,
+			_("Error writing to mailbox: %s. Mailbox NOT truncated: %s"),
 			mu_strerror (status), mu_strerror (rc));
 	  else  
-	    mailer_err (_("Error writing to mailbox: %s"),
+	    mailer_err ("write error", name,
+			_("Error writing to mailbox: %s"),
 	  	        mu_strerror (status));
 	}
     }
@@ -912,16 +991,22 @@ notify_biff (mu_mailbox_t mbox, char *name, size_t size)
 }
 
 void
-mailer_err (char *fmt, ...)
+mailer_err (const char *msg, const char *arg, const char *fmt, ...)
 {
   va_list ap;
 
   guess_retval (errno);
-  va_start (ap, fmt);
-  vfprintf (stderr, fmt, ap);
-  fprintf (stderr, "\n");
-  mu_verror (fmt, ap);
-  va_end (ap);
+  if (arg)
+    fprintf (stderr, "<%s>: %s", arg, msg);
+  else
+    fputs (msg, stderr);
+  fputc ('\n', stderr);
+  if (fmt)
+    {
+      va_start (ap, fmt);
+      mu_verror (fmt, ap);
+      va_end (ap);
+    }
 }
 
 int temp_errors[] = {
