@@ -1,9 +1,9 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2001, 2003, 2005 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2001, 2003, 2005, 2007 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
+   the Free Software Foundation; either version 3, or (at your option)
    any later version.
 
    GNU Mailutils is distributed in the hope that it will be useful,
@@ -35,9 +35,9 @@ mail_pipe (int argc, char **argv)
   off_t off = 0;
   size_t n = 0;
 
-  if (argc > 1)
+  if (argc > 2)
     cmd = argv[--argc];
-  else if (util_getenv (&cmd, "cmd", Mail_env_boolean, 1))
+  else if (util_getenv (&cmd, "cmd", Mail_env_string, 1))
     return 1;
 
   if (msgset_parse (argc, argv, MSG_NODELETED|MSG_SILENT, &list))
