@@ -17,13 +17,10 @@
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301 USA */
 
+#include "mail.h"
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
-#include <stdio.h>
-#include <getline.h>
-
-#include "mail.h"
 
 static int isfilename (const char *);
 static void msg_to_pipe (const char *cmd, mu_message_t msg);
@@ -514,7 +511,7 @@ mail_send0 (compose_env_t * env, int save_to)
   /* Prepare the header */
   if (util_getenv (NULL, "xmailer", Mail_env_boolean, 0) == 0)
     mu_header_set_value (env->header, MU_HEADER_X_MAILER, 
-                         argp_program_version, 1);
+                         program_version, 1);
 
   if (util_header_expand (&env->header) == 0)
     {
@@ -609,8 +606,7 @@ mail_send0 (compose_env_t * env, int save_to)
 			      mu_debug_t debug = NULL;
 			      mu_mailer_get_debug (mailer, &debug);
 			      mu_debug_set_level (debug,
-						  MU_DEBUG_TRACE | 
-                                                    MU_DEBUG_PROT);
+				         MU_DEBUG_LEVEL_UPTO (MU_DEBUG_PROT));
 			    }
 			  status = mu_mailer_open (mailer, MU_STREAM_RDWR);
 			  if (status == 0)
