@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
    Copyright (C) 1999, 2000, 2001, 2002, 2003, 
-   2004, 2005, 2006, 2007 Free Software Foundation, Inc.
+   2004, 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -76,12 +76,14 @@ static void amd_destroy (mu_mailbox_t mailbox);
 static int amd_open (mu_mailbox_t, int);
 static int amd_close (mu_mailbox_t);
 static int amd_get_message (mu_mailbox_t, size_t, mu_message_t *);
+static int amd_quick_get_message (mu_mailbox_t mailbox, mu_message_qid_t qid,
+				  mu_message_t *pmsg);
 static int amd_append_message (mu_mailbox_t, mu_message_t);
 static int amd_messages_count (mu_mailbox_t, size_t *);
 static int amd_messages_recent (mu_mailbox_t, size_t *);
 static int amd_message_unseen (mu_mailbox_t, size_t *);
 static int amd_expunge (mu_mailbox_t);
-static int amd_save_attributes (mu_mailbox_t);
+static int amd_sync (mu_mailbox_t);
 static int amd_uidnext (mu_mailbox_t mailbox, size_t *puidnext);
 static int amd_uidvalidity (mu_mailbox_t, unsigned long *);
 static int amd_scan (mu_mailbox_t, size_t, size_t *);
@@ -242,10 +244,11 @@ amd_array_shrink (struct _amd_data *amd, size_t index)
 
 
 int
-amd_init_mailbox (mu_mailbox_t mailbox, size_t amd_size, struct _amd_data **pamd) 
+amd_init_mailbox (mu_mailbox_t mailbox, size_t amd_size,
+		  struct _amd_data **pamd) 
 {
+  int status;
   struct _amd_data *amd;
-  size_t name_len;
 
   if (mailbox == NULL)
     return MU_ERR_MBX_NULL;
@@ -259,15 +262,13 @@ amd_init_mailbox (mu_mailbox_t mailbox, size_t amd_size, struct _amd_data **pamd
   /* Back pointer.  */
   amd->mailbox = mailbox;
 
-  mu_url_get_path (mailbox->url, NULL, 0, &name_len);
-  amd->name = calloc (name_len + 1, sizeof (char));
-  if (amd->name == NULL)
+  status = mu_url_aget_path (mailbox->url, &amd->name);
+  if (status)
     {
       free (amd);
       mailbox->data = NULL;
-      return ENOMEM;
+      return status;
     }
-  mu_url_get_path (mailbox->url, amd->name, name_len + 1, NULL);
 
   /* Overloading the defaults.  */
   mailbox->_destroy = amd_destroy;
@@ -277,12 +278,13 @@ amd_init_mailbox (mu_mailbox_t mailbox, size_t amd_size, struct _amd_data **pamd
 
   /* Overloading of the entire mailbox object methods.  */
   mailbox->_get_message = amd_get_message;
+  mailbox->_quick_get_message = amd_quick_get_message;
   mailbox->_append_message = amd_append_message;
   mailbox->_messages_count = amd_messages_count;
   mailbox->_messages_recent = amd_messages_recent;
   mailbox->_message_unseen = amd_message_unseen;
   mailbox->_expunge = amd_expunge;
-  mailbox->_save_attributes = amd_save_attributes;
+  mailbox->_sync = amd_sync;
   mailbox->_uidvalidity = amd_uidvalidity;
   mailbox->_uidnext = amd_uidnext;
 
@@ -291,7 +293,7 @@ amd_init_mailbox (mu_mailbox_t mailbox, size_t amd_size, struct _amd_data **pamd
 
   mailbox->_get_size = amd_get_size;
 
-  MAILBOX_DEBUG1 (mailbox, MU_DEBUG_TRACE, "amd_init(%s)\n", amd->name);
+  MU_DEBUG1 (mailbox->debug, MU_DEBUG_TRACE1, "amd_init(%s)\n", amd->name);
   *pamd = amd;
   return 0;
 }
@@ -333,10 +335,14 @@ amd_open (mu_mailbox_t mailbox, int flags)
     {
       if ((flags & MU_STREAM_CREAT) && errno == ENOENT)
 	{
+	  int rc;
+	  
 	  if (mkdir (amd->name, S_IRUSR|S_IWUSR|S_IXUSR))
 	    return errno;
 	  if (stat (amd->name, &st) < 0)
 	    return errno;
+	  if (amd->create && (rc = amd->create (amd, flags)))
+	    return rc;
 	}
       else
 	return errno;
@@ -345,8 +351,9 @@ amd_open (mu_mailbox_t mailbox, int flags)
   if (!S_ISDIR (st.st_mode))
     return EINVAL;
 
-  amd->mtime = st.st_mtime;
-
+  if (mailbox->locker == NULL)
+    mu_locker_create (&mailbox->locker, "/dev/null", 0);
+  
   return 0;
 }
 
@@ -381,6 +388,14 @@ amd_close (mu_mailbox_t mailbox)
   return 0;
 }
 
+static int
+amd_message_qid (mu_message_t msg, mu_message_qid_t *pqid)
+{
+  struct _amd_message *mhm = mu_message_get_owner (msg);
+  
+  return mhm->amd->cur_msg_file_name (mhm, pqid);
+}
+
 struct _amd_message *
 _amd_get_message (struct _amd_data *amd, size_t msgno)
 {
@@ -481,7 +496,8 @@ _amd_attach_message (mu_mailbox_t mailbox, struct _amd_message *mhm,
   /* Set the UID.  */
   if (mhm->amd->message_uid)
     mu_message_set_uid (msg, mhm->amd->message_uid, mhm);
-
+  mu_message_set_qid (msg, amd_message_qid, mhm);
+  
   /* Attach the message to the mailbox mbox data.  */
   mhm->message = msg;
   mu_message_set_mailbox (msg, mailbox, mhm);
@@ -518,6 +534,36 @@ amd_get_message (mu_mailbox_t mailbox, size_t msgno, mu_message_t *pmsg)
   return _amd_attach_message (mailbox, mhm, pmsg);
 }
 
+static int
+amd_quick_get_message (mu_mailbox_t mailbox, mu_message_qid_t qid,
+		       mu_message_t *pmsg)
+{
+  int status;
+  struct _amd_data *amd = mailbox->data;
+  if (amd->msg_count)
+    {
+      mu_message_qid_t vqid;
+      mu_message_t msg = amd->msg_array[0]->message;
+      status = mu_message_get_qid (msg, &vqid);
+      if (status)
+	return status;
+      status = strcmp (qid, vqid);
+      free (vqid);
+      if (status)
+	return MU_ERR_EXISTS;
+      *pmsg = msg;
+    }
+  else if (amd->qfetch)
+    {
+      status = amd->qfetch (amd, qid);
+      if (status)
+	return status;
+      return _amd_attach_message (mailbox, amd->msg_array[0], pmsg);
+    }
+  
+  return ENOSYS;
+}
+
 static FILE *
 _amd_tempfile(struct _amd_data *amd, char **namep)
 {
@@ -541,7 +587,8 @@ _amd_delim (char *str)
 }
 
 static int
-_amd_message_save (struct _amd_data *amd, struct _amd_message *mhm, int expunge)
+_amd_message_save (struct _amd_data *amd, struct _amd_message *mhm,
+		   int expunge)
 {
   mu_stream_t stream = NULL;
   char *name = NULL, *buf = NULL, *msg_name;
@@ -555,7 +602,7 @@ _amd_message_save (struct _amd_data *amd, struct _amd_message *mhm, int expunge)
   int status;
   mu_attribute_t attr;
   mu_body_t body;
-  char buffer[512];
+  const char *sbuf;
   mu_envelope_t env = NULL;
   
   status = mu_message_size (msg, &bsize);
@@ -599,7 +646,8 @@ _amd_message_save (struct _amd_data *amd, struct _amd_message *mhm, int expunge)
     }
 
   /* Add imapbase */
-  if (amd->next_uid
+  if (!(amd->mailbox->flags & MU_STREAM_APPEND)
+      && amd->next_uid
       && (!amd->msg_array || (amd->msg_array[0] == mhm))) /*FIXME*/
     {
       nbytes += fprintf (fp, "X-IMAPbase: %lu %u\n",
@@ -609,23 +657,22 @@ _amd_message_save (struct _amd_data *amd, struct _amd_message *mhm, int expunge)
     }
   
   mu_message_get_envelope (msg, &env);
-  if (mu_envelope_date (env, buffer, sizeof buffer, &n) == 0 && n > 0)
+  if (mu_envelope_sget_date (env, &sbuf) == 0)
     {
-      /* NOTE: buffer is terminated with \n */
-      char *p = buffer;
-      while (isspace (*p))
-	p++;
-      nbytes += fprintf (fp, "%s: %s", MU_HEADER_ENV_DATE, p);
+      /* NOTE: buffer might be terminated with \n */
+      while (*sbuf && isspace (*sbuf))
+	sbuf++;
+      nbytes += fprintf (fp, "%s: %s", MU_HEADER_ENV_DATE, sbuf);
 
-      if (*p && p[strlen (p) - 1] != '\n')
+      if (*sbuf && sbuf[strlen (sbuf) - 1] != '\n')
 	nbytes += fprintf (fp, "\n");
       
       nlines++;
     }
 	  
-  if (mu_envelope_sender (env, buffer, sizeof buffer, &n) == 0 && n > 0)
+  if (mu_envelope_sget_sender (env, &sbuf) == 0)
     {
-      fprintf (fp, "%s: %s\n", MU_HEADER_ENV_SENDER, buffer);
+      fprintf (fp, "%s: %s\n", MU_HEADER_ENV_SENDER, sbuf);
       nlines++;
     }
   
@@ -668,12 +715,29 @@ _amd_message_save (struct _amd_data *amd, struct _amd_message *mhm, int expunge)
   free (buf);
   fclose (fp);
 
-  msg_name = amd->msg_file_name (mhm, mhm->deleted);
-  rename (name, msg_name);
+  status = amd->new_msg_file_name (mhm, mhm->attr_flags, &msg_name);
+  if (status == 0)
+    {
+      char *old_name;
+      status = amd->cur_msg_file_name (mhm, &old_name);
+      if (status == 0)
+	{
+	  if (rename (name, msg_name))
+	    status = errno;
+	  else
+	    {
+	      if (strcmp (old_name, msg_name))
+		/* Unlink original message */
+		unlink (old_name);
+	    }
+	  free (old_name);
+	  mhm->orig_flags = mhm->attr_flags;
+	}
+      free (msg_name);
+    }
   free (name);
-  free (msg_name);
 
-  return 0;
+  return status;
 }
 
 static int
@@ -726,8 +790,19 @@ amd_append_message (mu_mailbox_t mailbox, mu_message_t msg)
     }
 
   if (amd->msg_finish_delivery)
-    amd->msg_finish_delivery (amd, mhm);
-      
+    status = amd->msg_finish_delivery (amd, mhm);
+
+  if (status == 0 && mailbox->observable)
+    {
+      char *qid;
+      if (amd->cur_msg_file_name (mhm, &qid) == 0)
+	{
+	  mu_observable_notify (mailbox->observable, MU_EVT_MESSAGE_APPEND,
+				qid);
+	  free (qid);
+	}
+    }
+  
   return status;
 }
 
@@ -829,10 +904,22 @@ amd_expunge (mu_mailbox_t mailbox)
       
       if (mhm->attr_flags & MU_ATTRIBUTE_DELETED)
 	{
-	  if (!mhm->deleted)
+	  if (!(mhm->orig_flags & MU_ATTRIBUTE_DELETED))
 	    {
-	      char *old_name = amd->msg_file_name (mhm, 0);
-	      char *new_name = amd->msg_file_name (mhm, 1);
+	      int rc;
+	      char *old_name;
+	      char *new_name;
+
+	      rc = amd->cur_msg_file_name (mhm, &old_name);
+	      if (rc)
+		return rc;
+	      rc = amd->new_msg_file_name (mhm, mhm->attr_flags, &new_name);
+	      if (rc)
+		{
+		  free (old_name);
+		  return rc;
+		}
+	      
 	      if (new_name)
 		{
 		  /* Rename original message */
@@ -853,7 +940,6 @@ amd_expunge (mu_mailbox_t mailbox)
 	      || (mhm->message && mu_message_is_modified (mhm->message)))
 	    {
 	      _amd_attach_message (mailbox, mhm, NULL);
-	      mhm->deleted = mhm->attr_flags & MU_ATTRIBUTE_DELETED;
 	      _amd_message_save (amd, mhm, 1);
 	    }
 	  i++; /* Move to the next message */
@@ -864,7 +950,7 @@ amd_expunge (mu_mailbox_t mailbox)
 }
 
 static int
-amd_save_attributes (mu_mailbox_t mailbox)
+amd_sync (mu_mailbox_t mailbox)
 {
   struct _amd_data *amd = mailbox->data;
   struct _amd_message *mhm;
@@ -893,7 +979,6 @@ amd_save_attributes (mu_mailbox_t mailbox)
 	  || (mhm->message && mu_message_is_modified (mhm->message)))
 	{
 	  _amd_attach_message (mailbox, mhm, NULL);
-	  mhm->deleted = mhm->attr_flags & MU_ATTRIBUTE_DELETED;
 	  _amd_message_save (amd, mhm, 0);
 	}
     }
@@ -1020,7 +1105,11 @@ amd_scan_message (struct _amd_message *mhm)
   if (mhm->mtime)
     {
       struct stat st;
-      char *msg_name = mhm->amd->msg_file_name (mhm, mhm->deleted);
+      char *msg_name;
+
+      status = mhm->amd->cur_msg_file_name (mhm, &msg_name);
+      if (status)
+	return status;
 
       if (stat (msg_name, &st) == 0 && st.st_mtime == mhm->mtime)
 	{
@@ -1097,9 +1186,6 @@ amd_is_updated (mu_mailbox_t mailbox)
   struct stat st;
   struct _amd_data *amd = mailbox->data;
 
-  if (amd->msg_count == 0)
-    return 0;
-
   if (stat (amd->name, &st) < 0)
     return 1;
 
@@ -1107,10 +1193,80 @@ amd_is_updated (mu_mailbox_t mailbox)
 }
 
 static int
-amd_get_size (mu_mailbox_t mailbox MU_ARG_UNUSED, mu_off_t *psize MU_ARG_UNUSED)
+compute_mailbox_size (const char *name, mu_off_t *psize)
 {
-  /*FIXME*/
-  return ENOSYS;
+  DIR *dir;
+  struct dirent *entry;
+  char *buf;
+  size_t bufsize;
+  size_t dirlen;
+  size_t flen;
+  int status = 0;
+  struct stat sb;
+  
+  dir = opendir (name);
+  if (!dir)
+    return errno;
+
+  dirlen = strlen (name);
+  bufsize = dirlen + 32;
+  buf = malloc (bufsize);
+  if (!buf)
+    {
+      closedir (dir);
+      return ENOMEM;
+    }
+  
+  strcpy (buf, name);
+  if (buf[dirlen-1] != '/')
+    buf[++dirlen - 1] = '/';
+	  
+  while ((entry = readdir (dir)))
+    {
+      switch (entry->d_name[0])
+	{
+	case '.':
+	  break;
+
+	default:
+	  flen = strlen (entry->d_name);
+	  if (dirlen + flen + 1 > bufsize)
+	    {
+	      bufsize = dirlen + flen + 1;
+	      buf = realloc (buf, bufsize);
+	      if (!buf)
+		{
+		  status = ENOMEM;
+		  break;
+		}
+	    }
+	  strcpy (buf + dirlen, entry->d_name);
+	  if (stat (buf, &sb) == 0)
+	    {
+	      if (S_ISREG (sb.st_mode))
+		*psize += sb.st_size;
+	      else if (S_ISDIR (sb.st_mode))
+		compute_mailbox_size (buf, psize);
+	    }
+	  /* FIXME: else? */
+	  break;
+	}
+    }
+
+  free (buf);
+  
+  closedir (dir);
+  return 0;
+}
+
+static int
+amd_get_size (mu_mailbox_t mailbox, mu_off_t *psize)
+{
+  struct _amd_data *amd = mailbox->data;
+  if (amd->mailbox_size)
+    return amd->mailbox_size (mailbox, psize);
+  *psize = 0;
+  return compute_mailbox_size (amd->name, psize);
 }
 
 /* Return number of open streams residing in a message pool */
@@ -1151,6 +1307,7 @@ amd_pool_lookup (struct _amd_message *mhm)
 static int
 amd_pool_open (struct _amd_message *mhm)
 {
+  int status;
   struct _amd_data *amd = mhm->amd;
   if (amd_pool_lookup (mhm))
     return 0;
@@ -1159,7 +1316,9 @@ amd_pool_open (struct _amd_message *mhm)
       amd_message_stream_close (amd->msg_pool[amd->pool_first++]);
       amd->pool_first %= MAX_OPEN_STREAMS;
     }
-  amd_message_stream_open (mhm);
+  status = amd_message_stream_open (mhm);
+  if (status)
+    return status;
   amd->msg_pool[amd->pool_last++] = mhm;
   amd->pool_last %= MAX_OPEN_STREAMS;
   return 0;
@@ -1186,12 +1345,13 @@ int
 amd_message_stream_open (struct _amd_message *mhm)
 {
   struct _amd_data *amd = mhm->amd;
-  char *filename = amd->msg_file_name (mhm, mhm->deleted);
+  char *filename;
   int status;
   int flags = MU_STREAM_ALLOW_LINKS;
-  
-  if (!filename)
-    return ENOMEM;
+
+  status = amd->cur_msg_file_name (mhm, &filename);
+  if (status)
+    return status;
 
   /* The message should be at least readable */
   if (amd->mailbox->flags & (MU_STREAM_RDWR|MU_STREAM_WRITE|MU_STREAM_APPEND))
@@ -1348,8 +1508,9 @@ amd_header_fill (mu_header_t header, char *buffer, size_t len,
 {
   mu_message_t msg = mu_header_get_owner (header);
   struct _amd_message *mhm = mu_message_get_owner (msg);
-
-  amd_pool_open (mhm);
+  int status = amd_pool_open (mhm);
+  if (status)
+    return status;
   return amd_readstream (mhm, buffer, len, off, pnread, 0,
 			 0, mhm->body_start);
 }
@@ -1395,7 +1556,7 @@ amd_unset_attr_flags (mu_attribute_t attr, int flags)
 /* Envelope */
 static int
 amd_envelope_date (mu_envelope_t envelope, char *buf, size_t len,
-		  size_t *psize)
+		   size_t *psize)
 {
   mu_message_t msg = mu_envelope_get_owner (envelope);
   struct _amd_message *mhm = mu_message_get_owner (msg);
@@ -1481,50 +1642,4 @@ amd_envelope_sender (mu_envelope_t envelope, char *buf, size_t len, size_t *psiz
   return 0;
 }
 
-static void
-amd_url_destroy (mu_url_t url MU_ARG_UNUSED)
-{
-}
-
-int
-amd_url_init (mu_url_t url, const char *scheme)
-{
-  const char *name = mu_url_to_string (url);
-  const char *path_ptr = name;
-  size_t len = strlen (name);
-  size_t scheme_len = strlen (scheme);
-    
-  if (!name)
-    return 0;
-
-  if (mu_scheme_autodetect_p (name, &path_ptr))
-    /* nothing */ ;
-  /* reject the obvious */
-  else if (strncmp (scheme, name, scheme_len) != 0
-	   || len < scheme_len + 1) 
-    return EINVAL;
-  else
-    path_ptr = name + scheme_len;
-  
-  /* TYPE */
-  url->_destroy = amd_url_destroy;
-
-  /* SCHEME */
-  url->scheme = strdup (scheme);
-  if (url->scheme == NULL)
-    {
-      amd_url_destroy (url);
-      return ENOMEM;
-    }
-
-  /* PATH */
-  url->path = strdup (path_ptr);
-  if (url->path == NULL)
-    {
-      amd_url_destroy (url);
-      return ENOMEM;
-    }
-
-  return 0;
-}
 
