@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
    Copyright (C) 1999, 2000, 2001, 2002, 2003,
-   2004, 2005, 2007 Free Software Foundation, Inc.
+   2004, 2005, 2007, 2008 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -27,6 +27,7 @@
 #endif
 
 #include <mailutils/mailutils.h>
+#include "mailutils/libargp.h"
 
 static int messages_count (const char *);
 
@@ -45,11 +46,10 @@ static struct argp_option options[] = {
 
 static const char *argp_capa[] = {
   "common",
+  "debug",
   "license",
   "mailbox",
-#ifdef WITH_TLS
-  "tls",
-#endif
+  "locking",
   NULL
 };
 
@@ -72,6 +72,7 @@ parse_opt (int key, char *arg, struct argp_state *state)
     case 's':
       silent = 1;
       break;
+      
     case ARGP_KEY_ARG:
       args->argv = realloc (args->argv,
 			    sizeof (char *) * (state->arg_num + 2));
@@ -79,6 +80,7 @@ parse_opt (int key, char *arg, struct argp_state *state)
       args->argv[state->arg_num + 1] = NULL;
       args->argc++;
       break;
+      
     default:
       return ARGP_ERR_UNKNOWN;
     }
@@ -102,17 +104,18 @@ main (int argc, char **argv)
   struct arguments args = {0, NULL};
 
   /* Native Language Support */
-  mu_init_nls ();
-
-  mu_argp_init (program_version, NULL);
-#ifdef WITH_TLS
-  mu_tls_init_client_argp ();
-#endif
-  mu_argp_parse (&argp, &argc, &argv, 0, argp_capa, NULL, &args);
+  MU_APP_INIT_NLS ();
 
   /* register the formats.  */
   mu_register_all_mbox_formats ();
 
+#ifdef WITH_TLS
+  mu_gocs_register ("tls", mu_tls_module_init);
+#endif
+  mu_argp_init (program_version, NULL);
+  if (mu_app_init (&argp, argp_capa, NULL, argc, argv, 0, NULL, &args))
+    exit (1);
+
   if (args.argc < 1 && messages_count (NULL) < 0)
     err = 1;
   else if (args.argc >= 1)
